@@ -4,7 +4,7 @@ import torch
 from torch import Tensor
 from torch.nn import Module, Linear, LayerNorm, Embedding
 
-from ..modules.modules import TransformerEncoder, TransformerDecoder, TanhHead
+from ..modules import TransformerEncoder, TransformerDecoder, TanhHead
 from ..utils import transposed_cat
 
 
@@ -63,7 +63,7 @@ class BartModel(Module):
 
         return self.decoder(
             tensor=x, decoder_mask=decoder_padding_mask, encoder_state=encoder_outputs, encoder_mask=attention_mask
-        )
+        )[0]
 
 
 class BartForSequenceClassification(Module):
