@@ -8,7 +8,7 @@ from torch import Tensor
 from torch.nn import Module, Linear
 from torch.nn.functional import gelu
 
-from ..modules.modules import TransformerEncoder, EmbeddingsWithTokenTypes
+from ..modules import TransformerEncoder, EmbeddingsWithTokenTypes
 from ..utils import transposed_cat
 
 
