@@ -20,12 +20,14 @@ class ClientSocket(SocketConnection):
 
     def __init__(self, connection: socket.socket, message_size: int):
         super().__init__(connection, message_size)
-        self.connection.setblocking(False)
         self.message_id: Optional[bytes] = None
         self.batch_ind: int = -1
 
     def empty(self):
-        self.connection.recv(self.message_size)
+        while True:
+            self.connection.recv(self.message_size)
+            if not select.select([self.connection], [], [], 0)[0]:
+                break
 
     def request_server(self):
         self.connection.send(Config.request_signal)
@@ -40,22 +42,16 @@ class ClientSocket(SocketConnection):
         self.batch_ind = batch_ind
 
     def receive_outputs(self) -> Optional[Tuple[int, Any]]:
-        self.connection.setblocking(True)
         data = self.read_buffer()
-        self.connection.setblocking(False)
         response_message_id = data[:Config.message_id_len]
         if response_message_id != self.message_id:
             logger.warning(f'Skipping response with wrong message id')
             return None
         data = data[Config.message_id_len:]
         self.message_id = None
-        outputs = (
-            self.batch_ind,
-            ConnectionError(data[len(Config.error_prefix):].decode('utf-8')) if data.startswith(Config.error_prefix)
-            else pkl.loads(data)
-        )
-        self.batch_ind = -1
-        return outputs
+        if data.startswith(Config.error_prefix):
+            raise ConnectionError(data[len(Config.error_prefix):].decode('utf-8'))
+        return self.batch_ind, pkl.loads(data)
 
 
 class ClientAPI:
@@ -122,9 +118,7 @@ class ClientAPI:
 
         client_sockets = self.client_sockets[thread_id][self.model]
 
-        buffered_sockets = select.select(client_sockets, [], [], 0)[0]
-
-        for client_socket in buffered_sockets:
+        for client_socket in select.select(client_sockets, [], [], 0)[0]:
             client_socket.empty()
 
         batched_inputs = self.split_inputs_to_batches(inputs, self.model_config.max_batch_size)
@@ -137,8 +131,11 @@ class ClientAPI:
             client_socket.request_server()
 
         while received_counter < len(outputs):
-
-            ready_sockets: List[ClientSocket] = select.select(client_sockets, [], [], None)[0]
+            ready_sockets: List[ClientSocket] = select.select(
+                client_sockets if batched_inputs else
+                [client_socket for client_socket in client_sockets if client_socket.message_id is not None],
+                [], [], None
+            )[0]
             input_sockets: List[ClientSocket] = []
             output_sockets: List[ClientSocket] = []
             for client_socket in ready_sockets:
@@ -155,7 +152,7 @@ class ClientAPI:
 
             batched_inputs = batched_inputs[added_connections:]
 
-            if not batched_inputs:
+            if added_connections and not batched_inputs:
                 for client_socket in client_sockets:
                     if client_socket.message_id is None:
                         client_socket.cancel_request()
